//+------------------------------------------------------------------+
//|                                                       EWO_EA.mq4 |
//|                        Copyright 2012, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright 2012, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"
#define MAGICMA  20050610

extern double Lots               = 0.1;
extern double MaximumRisk        = 0.02;
extern double DecreaseFactor     = 5;
extern double StopLoss           = 40;
extern double TakeProfit         = 0;

//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//---- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   if(lot<0.1) lot=0.1;
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   
   int    res;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;

// CONDITIONS:

double SL;
double TP;

//---- sell conditions : EWO cross below 0 then i check sar (above price) and then check ADX (50) - DI ABOVE + DI). STOP 40 PIPS.
   if(   iCustom(NULL,NULL,"EWO",0,0) < 0 
      && iSAR(NULL,NULL,0.02,0.2,0) > Ask
      && iADX(NULL,0,50,PRICE_CLOSE,MODE_MINUSDI,0) > iADX(NULL,0,50,PRICE_CLOSE,MODE_PLUSDI,0))
     {
      SL = NormalizeDouble(Ask+(StopLoss*Point),4);
      TP = NormalizeDouble(Bid-(TakeProfit*Point),4);
      if (StopLoss   == 0) SL = 0;
      if (TakeProfit == 0) TP = 0;
      res= OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,TP,"",MAGICMA,0,Red);
      return;
     }
//---- buy conditions : EWO cross above 0 then i check sar (below price) and then check ADX (50) + DI ABOVE - DI). STOP 40 PIPS.
   if(   iCustom(NULL,NULL,"EWO",0,0) > 0 
      && iSAR(NULL,NULL,0.02,0.2,0) < Bid
      && iADX(NULL,0,50,Point,MODE_PLUSDI,0) > iADX(NULL,0,50,PRICE_CLOSE,MODE_MINUSDI,0)) 
     {
      SL = NormalizeDouble(Bid-(StopLoss*Point),4);
      TP = NormalizeDouble(Ask+(TakeProfit*Point),4);
      if (StopLoss   == 0) SL = 0;
      if (TakeProfit == 0) TP = 0;
      res= OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,TP,"",MAGICMA,0,Blue);
      return;
     }
//----
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
 
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
 
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
      if(iCustom(NULL,NULL,"EWO",0,0)<0 
      && iCustom(NULL,NULL,"EWO",0,1)>=0
      && iSAR(NULL,NULL,0.02,0.2,0) > Ask
      && iADX(NULL,0,50,PRICE_HIGH,MODE_MINUSDI,0)>iADX(NULL,0,50,PRICE_HIGH,MODE_PLUSDI,0))
      OrderClose(OrderTicket(),OrderLots(),Bid,3,White);
         break;
        }
      if(OrderType()==OP_SELL)
        {
      if(iCustom(NULL,NULL,"EWO",0,0)>0 
      && iCustom(NULL,NULL,"EWO",0,1)<=0
      && iSAR(NULL,NULL,0.02,0.2,0) < Bid
      && iADX(NULL,0,50,PRICE_HIGH,MODE_PLUSDI,0)>iADX(NULL,0,50,PRICE_HIGH,MODE_MINUSDI,0)) 
      OrderClose(OrderTicket(),OrderLots(),Ask,3,White);
         break;
        }
     }
//----
  }
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
   else                                    CheckForClose();
//----
  }
//+------------------------------------------------------------------+